function cdf = truncated_normal_ab_cdf ( x, mu, s, a, b )

%*****************************************************************************80
%
%% TRUNCATED_NORMAL_AB_CDF evaluates the truncated Normal CDF.
%
%  Licensing:
%
%    This code is distributed under the GNU LGPL license.
%
%  Modified:
%
%    14 August 2013
%
%  Author:
%
%    John Burkardt
%
%  Parameters:
%
%    Input, real X, the argument of the CDF.
%
%    Input, real MU, S, the mean and standard deviation of the
%    parent Normal distribution.
%
%    Input, real A, B, the lower and upper truncation limits.
%
%    Output, real CDF, the value of the CDF.
%
  alpha = ( a - mu ) / s;
  beta = ( b - mu ) / s;
  xi = ( x - mu ) / s;

  alpha_cdf = normal_01_cdf ( alpha );
  beta_cdf = normal_01_cdf ( beta );
  xi_cdf = normal_01_cdf ( xi );
%   alpha_cdf = normcdf( alpha );
%   beta_cdf = normcdf( beta );
%   xi_cdf = normcdf( xi );  

  cdf = ( xi_cdf - alpha_cdf ) ./ ( beta_cdf - alpha_cdf );

  return
end
